from featurestore.core.schema import Column, Schema

from .base_job import BaseJob


class ExtractSchemaJob(BaseJob):
    def _response_method(self, job_id):
        response = self._stub.GetExtractSchemaJobOutput(job_id)
        return Schema(
            [Column(field.name, field.data_type) for field in response.schema]
        )



from copy import deepcopy

from . import CoreService_pb2 as pb
from .data_types import ARRAY, MAP
from .utils import extract_columns


class Column:
    def __init__(self, name, data_type):
        self.name = name
        if not isinstance(data_type, str):
            self.__data_type = self.to_string(data_type)
        else:
            self.__data_type = data_type

    @property
    def data_type(self):
        return self.__data_type

    @data_type.setter
    def data_type(self, value):
        self.__data_type = self.to_string(value)

    def to_string(self, data_type):
        if isinstance(data_type, (ARRAY, MAP)):
            return data_type.flatten()
        else:
            return pb.FeatureDataType.Name(data_type).lower()


class Schema:
    def __init__(self, columns):
        self.__order = [col.name for col in columns]
        self.__columns = {col.name: col for col in columns}

    @classmethod
    def load(cls, feature_set):
        fields = [
            Column(feature.name, feature.data_type)
            for feature in feature_set._feature_set.features
        ]
        return cls(fields)

    def dump(self):
        return [self._create_proto_spec(self.__columns[col]) for col in self.__order]

    def update_column(self, name, data_type):
        new_columns = deepcopy(self.__columns)
        new_columns[name].data_type = data_type
        return Schema([new_columns[name] for name in self.__order])

    def select(self, columns):
        new_cols = [self.__columns[name] for name in self.__order if name in columns]
        return Schema(new_cols)

    def exclude(self, columns):
        new_cols = [
            self.__columns[name] for name in self.__order if name not in columns
        ]
        return Schema(new_cols)

    def append(self, column, after=None):
        new_order = self.__order[:]
        if not after:
            new_order.append(column.name)
        else:
            index = new_order.index(after) + 1
            new_order.insert(index, column.name)
        new_cols = self.__columns.copy()
        new_cols[column.name] = column
        return Schema([new_cols[name] for name in new_order])

    def prepend(self, column, before=None):
        new_order = self.__order[:]
        if not before:
            new_order.insert(0, column.name)
        else:
            index = new_order.index(before)
            new_order.insert(index, column.name)
        new_cols = self.__columns.copy()
        new_cols[column.name] = column
        return Schema([new_cols[name] for name in new_order])

    @classmethod
    def from_string(cls, schema_str):
        return cls(extract_columns(schema_str))

    def to_string(self):
        schema = []
        for fld in self.__order:
            col = self.__columns[fld]
            schema.append(f"{col.name} {col.data_type}")
        return ", ".join(schema)

    def _create_proto_spec(self, column):
        feature_spec = pb.FeatureSchema()
        feature_spec.name = column.name
        feature_spec.data_type = column.data_type
        return feature_spec

    def __repr__(self):
        return str(
            [
                {
                    "name": self.__columns[col].name,
                    "data_type": self.__columns[col].data_type,
                }
                for col in self.__order
            ]
        )



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from featurestore.core import CoreService_pb2 as featurestore_dot_core_dot_CoreService__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


class CoreServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ListJobs = channel.unary_unary(
                '/featurestore.core.CoreService/ListJobs',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.ListJobsRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.ListJobsResponse.FromString,
                )
        self.GetJob = channel.unary_unary(
                '/featurestore.core.CoreService/GetJob',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.JobId.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.Job.FromString,
                )
        self.GetJobProgress = channel.unary_unary(
                '/featurestore.core.CoreService/GetJobProgress',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.JobProgressInput.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.JobProgressOutput.FromString,
                )
        self.StartExtractSchemaJob = channel.unary_unary(
                '/featurestore.core.CoreService/StartExtractSchemaJob',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.StartExtractSchemaJobRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.JobId.FromString,
                )
        self.GetExtractSchemaJobOutput = channel.unary_unary(
                '/featurestore.core.CoreService/GetExtractSchemaJobOutput',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.JobId.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.ExtractSchemaResponse.FromString,
                )
        self.StartIngestJob = channel.unary_unary(
                '/featurestore.core.CoreService/StartIngestJob',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.StartIngestJobRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.JobId.FromString,
                )
        self.GetIngestJobOutput = channel.unary_unary(
                '/featurestore.core.CoreService/GetIngestJobOutput',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.JobId.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.IngestResponse.FromString,
                )
        self.StartRetrieveAsLinksJob = channel.unary_unary(
                '/featurestore.core.CoreService/StartRetrieveAsLinksJob',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.RetrieveRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.JobId.FromString,
                )
        self.GetRetrieveAsLinksJobOutput = channel.unary_unary(
                '/featurestore.core.CoreService/GetRetrieveAsLinksJobOutput',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.JobId.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.RetrieveAsLinksResponse.FromString,
                )
        self.RetrieveAsSpark = channel.unary_unary(
                '/featurestore.core.CoreService/RetrieveAsSpark',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.RetrieveRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.RetrieveAsSparkResponse.FromString,
                )
        self.StartPreviewJob = channel.unary_unary(
                '/featurestore.core.CoreService/StartPreviewJob',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.PreviewRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.JobId.FromString,
                )
        self.GetPreviewJobOutput = channel.unary_unary(
                '/featurestore.core.CoreService/GetPreviewJobOutput',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.JobId.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.PreviewResponse.FromString,
                )
        self.GetIngestWriteCredentials = channel.unary_unary(
                '/featurestore.core.CoreService/GetIngestWriteCredentials',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.IngestWriteCredentialRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.IngestWriteCredentialResponse.FromString,
                )
        self.StartRevertIngestJob = channel.unary_unary(
                '/featurestore.core.CoreService/StartRevertIngestJob',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.StartRevertIngestJobRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.JobId.FromString,
                )
        self.GetRevertIngestJobOutput = channel.unary_unary(
                '/featurestore.core.CoreService/GetRevertIngestJobOutput',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.JobId.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.HasPermissionToRetrieve = channel.unary_unary(
                '/featurestore.core.CoreService/HasPermissionToRetrieve',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.HasPermissionToRetrieveRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.HasPermissionToRetrieveResponse.FromString,
                )
        self.RegisterFeatureSet = channel.unary_unary(
                '/featurestore.core.CoreService/RegisterFeatureSet',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.RegisterFeatureSetRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.FeatureSetResponse.FromString,
                )
        self.CreateNewFeatureSetVersion = channel.unary_unary(
                '/featurestore.core.CoreService/CreateNewFeatureSetVersion',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.CreateNewFeatureSetVersionRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.FeatureSetResponse.FromString,
                )
        self.GetOnlineRetrieveMeta = channel.unary_unary(
                '/featurestore.core.CoreService/GetOnlineRetrieveMeta',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.OnlineRetrieveMeta.FromString,
                )
        self.UpdateFeatureSetMetadata = channel.unary_unary(
                '/featurestore.core.CoreService/UpdateFeatureSetMetadata',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.UpdateFeatureSetMetadataRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.FeatureSetResponse.FromString,
                )
        self.GetVersion = channel.unary_unary(
                '/featurestore.core.CoreService/GetVersion',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.GetVersionResponse.FromString,
                )
        self.GetFeatureSet = channel.unary_unary(
                '/featurestore.core.CoreService/GetFeatureSet',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.GetFeatureSetRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.FeatureSetResponse.FromString,
                )
        self.ListFeatureSetVersions = channel.unary_unary(
                '/featurestore.core.CoreService/ListFeatureSetVersions',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.ListFeatureSetsVersionRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.ListFeatureSetVersionsResponse.FromString,
                )
        self.DeleteFeatureSet = channel.unary_unary(
                '/featurestore.core.CoreService/DeleteFeatureSet',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.DeleteFeatureSetRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.ListFeatureSets = channel.unary_unary(
                '/featurestore.core.CoreService/ListFeatureSets',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.ListFeatureSetsRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.ListFeatureSetsResponse.FromString,
                )
        self.FeatureSetExists = channel.unary_unary(
                '/featurestore.core.CoreService/FeatureSetExists',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.FeatureSetExistsRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.ExistsResponse.FromString,
                )
        self.GetIngestHistory = channel.unary_unary(
                '/featurestore.core.CoreService/GetIngestHistory',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.GetIngestHistoryRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.GetIngestHistoryResponse.FromString,
                )
        self.ListProjects = channel.unary_unary(
                '/featurestore.core.CoreService/ListProjects',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.ListProjectsResponse.FromString,
                )
        self.CreateProject = channel.unary_unary(
                '/featurestore.core.CoreService/CreateProject',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.CreateProjectRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.CreateProjectResponse.FromString,
                )
        self.GetProject = channel.unary_unary(
                '/featurestore.core.CoreService/GetProject',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.GetProjectRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.ProjectResponse.FromString,
                )
        self.DeleteProject = channel.unary_unary(
                '/featurestore.core.CoreService/DeleteProject',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.DeleteProjectRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.UpdateProjectMetadata = channel.unary_unary(
                '/featurestore.core.CoreService/UpdateProjectMetadata',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.UpdateProjectMetadataRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.ProjectResponse.FromString,
                )
        self.GetProjectsDefault = channel.unary_unary(
                '/featurestore.core.CoreService/GetProjectsDefault',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.ProjectsDefaultResponse.FromString,
                )
        self.ProjectExists = channel.unary_unary(
                '/featurestore.core.CoreService/ProjectExists',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.ProjectExistsRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.ExistsResponse.FromString,
                )
        self.GetAccessToken = channel.unary_unary(
                '/featurestore.core.CoreService/GetAccessToken',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.RefreshTokenRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.AccessTokenResponse.FromString,
                )
        self.GetUserByMail = channel.unary_unary(
                '/featurestore.core.CoreService/GetUserByMail',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.GetUserByMailRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.GetUserByMailResponse.FromString,
                )
        self.Logout = channel.unary_unary(
                '/featurestore.core.CoreService/Logout',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.LogoutRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.GetActiveUser = channel.unary_unary(
                '/featurestore.core.CoreService/GetActiveUser',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.GetActiveUserResponse.FromString,
                )
        self.Login = channel.unary_stream(
                '/featurestore.core.CoreService/Login',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.LoginResponse.FromString,
                )
        self.AddFeatureSetPermission = channel.unary_unary(
                '/featurestore.core.CoreService/AddFeatureSetPermission',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.FeatureSetPermissionRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.FeatureSetResponse.FromString,
                )
        self.AddProjectPermission = channel.unary_unary(
                '/featurestore.core.CoreService/AddProjectPermission',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.ProjectPermissionRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.ProjectResponse.FromString,
                )
        self.RemoveFeatureSetPermission = channel.unary_unary(
                '/featurestore.core.CoreService/RemoveFeatureSetPermission',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.FeatureSetPermissionRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.FeatureSetResponse.FromString,
                )
        self.RemoveProjectPermission = channel.unary_unary(
                '/featurestore.core.CoreService/RemoveProjectPermission',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.ProjectPermissionRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.ProjectResponse.FromString,
                )
        self.RevokeToken = channel.unary_unary(
                '/featurestore.core.CoreService/RevokeToken',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.TokenRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.GenerateToken = channel.unary_unary(
                '/featurestore.core.CoreService/GenerateToken',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.GenerateTokenRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.RawTokenResponse.FromString,
                )
        self.ListTokens = channel.unary_unary(
                '/featurestore.core.CoreService/ListTokens',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.ListTokensResponse.FromString,
                )
        self.GetToken = channel.unary_unary(
                '/featurestore.core.CoreService/GetToken',
                request_serializer=featurestore_dot_core_dot_CoreService__pb2.TokenRequest.SerializeToString,
                response_deserializer=featurestore_dot_core_dot_CoreService__pb2.TokenResponse.FromString,
                )


class CoreServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def ListJobs(self, request, context):
        """Jobs API
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetJob(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetJobProgress(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StartExtractSchemaJob(self, request, context):
        """Extract Schema API
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetExtractSchemaJobOutput(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StartIngestJob(self, request, context):
        """Feature Set API
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetIngestJobOutput(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StartRetrieveAsLinksJob(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetRetrieveAsLinksJobOutput(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RetrieveAsSpark(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StartPreviewJob(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPreviewJobOutput(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetIngestWriteCredentials(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StartRevertIngestJob(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetRevertIngestJobOutput(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def HasPermissionToRetrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RegisterFeatureSet(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateNewFeatureSetVersion(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetOnlineRetrieveMeta(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateFeatureSetMetadata(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetVersion(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetFeatureSet(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListFeatureSetVersions(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteFeatureSet(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListFeatureSets(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def FeatureSetExists(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetIngestHistory(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListProjects(self, request, context):
        """Project API
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateProject(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetProject(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteProject(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateProjectMetadata(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetProjectsDefault(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ProjectExists(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAccessToken(self, request, context):
        """Auth API
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetUserByMail(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Logout(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetActiveUser(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Login(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AddFeatureSetPermission(self, request, context):
        """Permissions API
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AddProjectPermission(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RemoveFeatureSetPermission(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RemoveProjectPermission(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RevokeToken(self, request, context):
        """PAT API
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GenerateToken(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListTokens(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetToken(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_CoreServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'ListJobs': grpc.unary_unary_rpc_method_handler(
                    servicer.ListJobs,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.ListJobsRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.ListJobsResponse.SerializeToString,
            ),
            'GetJob': grpc.unary_unary_rpc_method_handler(
                    servicer.GetJob,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.JobId.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.Job.SerializeToString,
            ),
            'GetJobProgress': grpc.unary_unary_rpc_method_handler(
                    servicer.GetJobProgress,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.JobProgressInput.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.JobProgressOutput.SerializeToString,
            ),
            'StartExtractSchemaJob': grpc.unary_unary_rpc_method_handler(
                    servicer.StartExtractSchemaJob,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.StartExtractSchemaJobRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.JobId.SerializeToString,
            ),
            'GetExtractSchemaJobOutput': grpc.unary_unary_rpc_method_handler(
                    servicer.GetExtractSchemaJobOutput,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.JobId.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.ExtractSchemaResponse.SerializeToString,
            ),
            'StartIngestJob': grpc.unary_unary_rpc_method_handler(
                    servicer.StartIngestJob,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.StartIngestJobRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.JobId.SerializeToString,
            ),
            'GetIngestJobOutput': grpc.unary_unary_rpc_method_handler(
                    servicer.GetIngestJobOutput,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.JobId.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.IngestResponse.SerializeToString,
            ),
            'StartRetrieveAsLinksJob': grpc.unary_unary_rpc_method_handler(
                    servicer.StartRetrieveAsLinksJob,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.RetrieveRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.JobId.SerializeToString,
            ),
            'GetRetrieveAsLinksJobOutput': grpc.unary_unary_rpc_method_handler(
                    servicer.GetRetrieveAsLinksJobOutput,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.JobId.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.RetrieveAsLinksResponse.SerializeToString,
            ),
            'RetrieveAsSpark': grpc.unary_unary_rpc_method_handler(
                    servicer.RetrieveAsSpark,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.RetrieveRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.RetrieveAsSparkResponse.SerializeToString,
            ),
            'StartPreviewJob': grpc.unary_unary_rpc_method_handler(
                    servicer.StartPreviewJob,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.PreviewRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.JobId.SerializeToString,
            ),
            'GetPreviewJobOutput': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPreviewJobOutput,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.JobId.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.PreviewResponse.SerializeToString,
            ),
            'GetIngestWriteCredentials': grpc.unary_unary_rpc_method_handler(
                    servicer.GetIngestWriteCredentials,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.IngestWriteCredentialRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.IngestWriteCredentialResponse.SerializeToString,
            ),
            'StartRevertIngestJob': grpc.unary_unary_rpc_method_handler(
                    servicer.StartRevertIngestJob,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.StartRevertIngestJobRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.JobId.SerializeToString,
            ),
            'GetRevertIngestJobOutput': grpc.unary_unary_rpc_method_handler(
                    servicer.GetRevertIngestJobOutput,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.JobId.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'HasPermissionToRetrieve': grpc.unary_unary_rpc_method_handler(
                    servicer.HasPermissionToRetrieve,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.HasPermissionToRetrieveRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.HasPermissionToRetrieveResponse.SerializeToString,
            ),
            'RegisterFeatureSet': grpc.unary_unary_rpc_method_handler(
                    servicer.RegisterFeatureSet,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.RegisterFeatureSetRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.FeatureSetResponse.SerializeToString,
            ),
            'CreateNewFeatureSetVersion': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateNewFeatureSetVersion,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.CreateNewFeatureSetVersionRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.FeatureSetResponse.SerializeToString,
            ),
            'GetOnlineRetrieveMeta': grpc.unary_unary_rpc_method_handler(
                    servicer.GetOnlineRetrieveMeta,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.OnlineRetrieveMeta.SerializeToString,
            ),
            'UpdateFeatureSetMetadata': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdateFeatureSetMetadata,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.UpdateFeatureSetMetadataRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.FeatureSetResponse.SerializeToString,
            ),
            'GetVersion': grpc.unary_unary_rpc_method_handler(
                    servicer.GetVersion,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.GetVersionResponse.SerializeToString,
            ),
            'GetFeatureSet': grpc.unary_unary_rpc_method_handler(
                    servicer.GetFeatureSet,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.GetFeatureSetRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.FeatureSetResponse.SerializeToString,
            ),
            'ListFeatureSetVersions': grpc.unary_unary_rpc_method_handler(
                    servicer.ListFeatureSetVersions,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.ListFeatureSetsVersionRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.ListFeatureSetVersionsResponse.SerializeToString,
            ),
            'DeleteFeatureSet': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteFeatureSet,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.DeleteFeatureSetRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'ListFeatureSets': grpc.unary_unary_rpc_method_handler(
                    servicer.ListFeatureSets,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.ListFeatureSetsRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.ListFeatureSetsResponse.SerializeToString,
            ),
            'FeatureSetExists': grpc.unary_unary_rpc_method_handler(
                    servicer.FeatureSetExists,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.FeatureSetExistsRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.ExistsResponse.SerializeToString,
            ),
            'GetIngestHistory': grpc.unary_unary_rpc_method_handler(
                    servicer.GetIngestHistory,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.GetIngestHistoryRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.GetIngestHistoryResponse.SerializeToString,
            ),
            'ListProjects': grpc.unary_unary_rpc_method_handler(
                    servicer.ListProjects,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.ListProjectsResponse.SerializeToString,
            ),
            'CreateProject': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateProject,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.CreateProjectRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.CreateProjectResponse.SerializeToString,
            ),
            'GetProject': grpc.unary_unary_rpc_method_handler(
                    servicer.GetProject,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.GetProjectRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.ProjectResponse.SerializeToString,
            ),
            'DeleteProject': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteProject,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.DeleteProjectRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'UpdateProjectMetadata': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdateProjectMetadata,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.UpdateProjectMetadataRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.ProjectResponse.SerializeToString,
            ),
            'GetProjectsDefault': grpc.unary_unary_rpc_method_handler(
                    servicer.GetProjectsDefault,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.ProjectsDefaultResponse.SerializeToString,
            ),
            'ProjectExists': grpc.unary_unary_rpc_method_handler(
                    servicer.ProjectExists,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.ProjectExistsRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.ExistsResponse.SerializeToString,
            ),
            'GetAccessToken': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAccessToken,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.RefreshTokenRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.AccessTokenResponse.SerializeToString,
            ),
            'GetUserByMail': grpc.unary_unary_rpc_method_handler(
                    servicer.GetUserByMail,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.GetUserByMailRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.GetUserByMailResponse.SerializeToString,
            ),
            'Logout': grpc.unary_unary_rpc_method_handler(
                    servicer.Logout,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.LogoutRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'GetActiveUser': grpc.unary_unary_rpc_method_handler(
                    servicer.GetActiveUser,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.GetActiveUserResponse.SerializeToString,
            ),
            'Login': grpc.unary_stream_rpc_method_handler(
                    servicer.Login,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.LoginResponse.SerializeToString,
            ),
            'AddFeatureSetPermission': grpc.unary_unary_rpc_method_handler(
                    servicer.AddFeatureSetPermission,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.FeatureSetPermissionRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.FeatureSetResponse.SerializeToString,
            ),
            'AddProjectPermission': grpc.unary_unary_rpc_method_handler(
                    servicer.AddProjectPermission,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.ProjectPermissionRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.ProjectResponse.SerializeToString,
            ),
            'RemoveFeatureSetPermission': grpc.unary_unary_rpc_method_handler(
                    servicer.RemoveFeatureSetPermission,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.FeatureSetPermissionRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.FeatureSetResponse.SerializeToString,
            ),
            'RemoveProjectPermission': grpc.unary_unary_rpc_method_handler(
                    servicer.RemoveProjectPermission,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.ProjectPermissionRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.ProjectResponse.SerializeToString,
            ),
            'RevokeToken': grpc.unary_unary_rpc_method_handler(
                    servicer.RevokeToken,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.TokenRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'GenerateToken': grpc.unary_unary_rpc_method_handler(
                    servicer.GenerateToken,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.GenerateTokenRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.RawTokenResponse.SerializeToString,
            ),
            'ListTokens': grpc.unary_unary_rpc_method_handler(
                    servicer.ListTokens,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.ListTokensResponse.SerializeToString,
            ),
            'GetToken': grpc.unary_unary_rpc_method_handler(
                    servicer.GetToken,
                    request_deserializer=featurestore_dot_core_dot_CoreService__pb2.TokenRequest.FromString,
                    response_serializer=featurestore_dot_core_dot_CoreService__pb2.TokenResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'featurestore.core.CoreService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class CoreService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def ListJobs(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/ListJobs',
            featurestore_dot_core_dot_CoreService__pb2.ListJobsRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.ListJobsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetJob(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/GetJob',
            featurestore_dot_core_dot_CoreService__pb2.JobId.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.Job.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetJobProgress(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/GetJobProgress',
            featurestore_dot_core_dot_CoreService__pb2.JobProgressInput.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.JobProgressOutput.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StartExtractSchemaJob(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/StartExtractSchemaJob',
            featurestore_dot_core_dot_CoreService__pb2.StartExtractSchemaJobRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.JobId.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetExtractSchemaJobOutput(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/GetExtractSchemaJobOutput',
            featurestore_dot_core_dot_CoreService__pb2.JobId.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.ExtractSchemaResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StartIngestJob(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/StartIngestJob',
            featurestore_dot_core_dot_CoreService__pb2.StartIngestJobRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.JobId.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetIngestJobOutput(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/GetIngestJobOutput',
            featurestore_dot_core_dot_CoreService__pb2.JobId.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.IngestResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StartRetrieveAsLinksJob(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/StartRetrieveAsLinksJob',
            featurestore_dot_core_dot_CoreService__pb2.RetrieveRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.JobId.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetRetrieveAsLinksJobOutput(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/GetRetrieveAsLinksJobOutput',
            featurestore_dot_core_dot_CoreService__pb2.JobId.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.RetrieveAsLinksResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RetrieveAsSpark(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/RetrieveAsSpark',
            featurestore_dot_core_dot_CoreService__pb2.RetrieveRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.RetrieveAsSparkResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StartPreviewJob(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/StartPreviewJob',
            featurestore_dot_core_dot_CoreService__pb2.PreviewRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.JobId.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPreviewJobOutput(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/GetPreviewJobOutput',
            featurestore_dot_core_dot_CoreService__pb2.JobId.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.PreviewResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetIngestWriteCredentials(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/GetIngestWriteCredentials',
            featurestore_dot_core_dot_CoreService__pb2.IngestWriteCredentialRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.IngestWriteCredentialResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StartRevertIngestJob(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/StartRevertIngestJob',
            featurestore_dot_core_dot_CoreService__pb2.StartRevertIngestJobRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.JobId.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetRevertIngestJobOutput(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/GetRevertIngestJobOutput',
            featurestore_dot_core_dot_CoreService__pb2.JobId.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def HasPermissionToRetrieve(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/HasPermissionToRetrieve',
            featurestore_dot_core_dot_CoreService__pb2.HasPermissionToRetrieveRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.HasPermissionToRetrieveResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RegisterFeatureSet(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/RegisterFeatureSet',
            featurestore_dot_core_dot_CoreService__pb2.RegisterFeatureSetRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.FeatureSetResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateNewFeatureSetVersion(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/CreateNewFeatureSetVersion',
            featurestore_dot_core_dot_CoreService__pb2.CreateNewFeatureSetVersionRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.FeatureSetResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetOnlineRetrieveMeta(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/GetOnlineRetrieveMeta',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.OnlineRetrieveMeta.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UpdateFeatureSetMetadata(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/UpdateFeatureSetMetadata',
            featurestore_dot_core_dot_CoreService__pb2.UpdateFeatureSetMetadataRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.FeatureSetResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetVersion(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/GetVersion',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.GetVersionResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetFeatureSet(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/GetFeatureSet',
            featurestore_dot_core_dot_CoreService__pb2.GetFeatureSetRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.FeatureSetResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListFeatureSetVersions(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/ListFeatureSetVersions',
            featurestore_dot_core_dot_CoreService__pb2.ListFeatureSetsVersionRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.ListFeatureSetVersionsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteFeatureSet(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/DeleteFeatureSet',
            featurestore_dot_core_dot_CoreService__pb2.DeleteFeatureSetRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListFeatureSets(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/ListFeatureSets',
            featurestore_dot_core_dot_CoreService__pb2.ListFeatureSetsRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.ListFeatureSetsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def FeatureSetExists(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/FeatureSetExists',
            featurestore_dot_core_dot_CoreService__pb2.FeatureSetExistsRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.ExistsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetIngestHistory(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/GetIngestHistory',
            featurestore_dot_core_dot_CoreService__pb2.GetIngestHistoryRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.GetIngestHistoryResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListProjects(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/ListProjects',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.ListProjectsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateProject(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/CreateProject',
            featurestore_dot_core_dot_CoreService__pb2.CreateProjectRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.CreateProjectResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetProject(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/GetProject',
            featurestore_dot_core_dot_CoreService__pb2.GetProjectRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.ProjectResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteProject(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/DeleteProject',
            featurestore_dot_core_dot_CoreService__pb2.DeleteProjectRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UpdateProjectMetadata(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/UpdateProjectMetadata',
            featurestore_dot_core_dot_CoreService__pb2.UpdateProjectMetadataRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.ProjectResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetProjectsDefault(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/GetProjectsDefault',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.ProjectsDefaultResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ProjectExists(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/ProjectExists',
            featurestore_dot_core_dot_CoreService__pb2.ProjectExistsRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.ExistsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetAccessToken(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/GetAccessToken',
            featurestore_dot_core_dot_CoreService__pb2.RefreshTokenRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.AccessTokenResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetUserByMail(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/GetUserByMail',
            featurestore_dot_core_dot_CoreService__pb2.GetUserByMailRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.GetUserByMailResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Logout(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/Logout',
            featurestore_dot_core_dot_CoreService__pb2.LogoutRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetActiveUser(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/GetActiveUser',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.GetActiveUserResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Login(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/featurestore.core.CoreService/Login',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.LoginResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AddFeatureSetPermission(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/AddFeatureSetPermission',
            featurestore_dot_core_dot_CoreService__pb2.FeatureSetPermissionRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.FeatureSetResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AddProjectPermission(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/AddProjectPermission',
            featurestore_dot_core_dot_CoreService__pb2.ProjectPermissionRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.ProjectResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RemoveFeatureSetPermission(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/RemoveFeatureSetPermission',
            featurestore_dot_core_dot_CoreService__pb2.FeatureSetPermissionRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.FeatureSetResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RemoveProjectPermission(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/RemoveProjectPermission',
            featurestore_dot_core_dot_CoreService__pb2.ProjectPermissionRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.ProjectResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RevokeToken(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/RevokeToken',
            featurestore_dot_core_dot_CoreService__pb2.TokenRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GenerateToken(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/GenerateToken',
            featurestore_dot_core_dot_CoreService__pb2.GenerateTokenRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.RawTokenResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListTokens(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/ListTokens',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.ListTokensResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetToken(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/featurestore.core.CoreService/GetToken',
            featurestore_dot_core_dot_CoreService__pb2.TokenRequest.SerializeToString,
            featurestore_dot_core_dot_CoreService__pb2.TokenResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from . import CoreService_pb2 as pb

TINYINT = pb.FeatureDataType.TinyInt
SMALLINT = pb.FeatureDataType.SmallInt
BIGINT = pb.FeatureDataType.BigInt
INT = pb.FeatureDataType.Int
DOUBLE = pb.FeatureDataType.Double
FLOAT = pb.FeatureDataType.Float
STRING = pb.FeatureDataType.String
BINARY = pb.FeatureDataType.Binary
BOOLEAN = pb.FeatureDataType.Boolean
DATE = pb.FeatureDataType.Date
TIMESTAMP = pb.FeatureDataType.Timestamp


class ARRAY:
    """ARRAY type accepts the type of the element as an argument"""

    def __init__(self, sub_type):
        try:
            self.sub_type = pb.FeatureDataType.Name(sub_type).lower()
        except:
            self.sub_type = sub_type.flatten()

    def flatten(self):
        return f"array<{self.sub_type}>"


class MAP:
    """STRUCT type accepts the type of the key and the value as arguments"""

    def __init__(self, key_type, value_type):
        try:
            self.key_type = pb.FeatureDataType.Name(key_type).lower()
        except:
            self.key_type = key_type.flatten()
        try:
            self.value_type = pb.FeatureDataType.Name(value_type).lower()
        except:
            self.value_type = value_type.flatten()

    def flatten(self):
        return f"map<{self.key_type},{self.value_type}>"



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: featurestore/core/FeatureSetSearch.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='featurestore/core/FeatureSetSearch.proto',
  package='featurestore.core',
  syntax='proto3',
  serialized_options=b'\n\030ai.h2o.featurestore.coreB\025FeatureSetSearchProto',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n(featurestore/core/FeatureSetSearch.proto\x12\x11\x66\x65\x61turestore.core\"?\n\rTextualFilter\x12\r\n\x05\x66ield\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x03(\t\x12\x10\n\x08operator\x18\x03 \x01(\t\"T\n\x0fNumericalFilter\x12\r\n\x05\x66ield\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x01\x12\x10\n\x08operator\x18\x03 \x01(\t\x12\x11\n\taggregate\x18\x04 \x01(\t\"?\n\rBooleanFilter\x12\r\n\x05\x66ield\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08\x12\x10\n\x08operator\x18\x03 \x01(\t\"\xdd\x01\n\x0f\x43ountCaseFilter\x12\x30\n\x04text\x18\x01 \x01(\x0b\x32 .featurestore.core.TextualFilterH\x00\x12\x35\n\x07numeric\x18\x02 \x01(\x0b\x32\".featurestore.core.NumericalFilterH\x00\x12\x33\n\x07\x62oolean\x18\x03 \x01(\x0b\x32 .featurestore.core.BooleanFilterH\x00\x12\x10\n\x08operator\x18\x04 \x01(\t\x12\r\n\x05value\x18\x05 \x01(\x05\x42\x0b\n\tcondition\"\xed\x01\n\x06\x46ilter\x12\x30\n\x04text\x18\x01 \x01(\x0b\x32 .featurestore.core.TextualFilterH\x00\x12\x35\n\x07numeric\x18\x02 \x01(\x0b\x32\".featurestore.core.NumericalFilterH\x00\x12\x33\n\x07\x62oolean\x18\x03 \x01(\x0b\x32 .featurestore.core.BooleanFilterH\x00\x12\x38\n\ncount_case\x18\x04 \x01(\x0b\x32\".featurestore.core.CountCaseFilterH\x00\x42\x0b\n\tcondition\"3\n\x05Query\x12*\n\x07\x66ilters\x18\x01 \x03(\x0b\x32\x19.featurestore.core.FilterB1\n\x18\x61i.h2o.featurestore.coreB\x15\x46\x65\x61tureSetSearchProtob\x06proto3'
)




_TEXTUALFILTER = _descriptor.Descriptor(
  name='TextualFilter',
  full_name='featurestore.core.TextualFilter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='field', full_name='featurestore.core.TextualFilter.field', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='featurestore.core.TextualFilter.value', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='operator', full_name='featurestore.core.TextualFilter.operator', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=63,
  serialized_end=126,
)


_NUMERICALFILTER = _descriptor.Descriptor(
  name='NumericalFilter',
  full_name='featurestore.core.NumericalFilter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='field', full_name='featurestore.core.NumericalFilter.field', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='featurestore.core.NumericalFilter.value', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='operator', full_name='featurestore.core.NumericalFilter.operator', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='aggregate', full_name='featurestore.core.NumericalFilter.aggregate', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=128,
  serialized_end=212,
)


_BOOLEANFILTER = _descriptor.Descriptor(
  name='BooleanFilter',
  full_name='featurestore.core.BooleanFilter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='field', full_name='featurestore.core.BooleanFilter.field', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='featurestore.core.BooleanFilter.value', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='operator', full_name='featurestore.core.BooleanFilter.operator', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=214,
  serialized_end=277,
)


_COUNTCASEFILTER = _descriptor.Descriptor(
  name='CountCaseFilter',
  full_name='featurestore.core.CountCaseFilter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='text', full_name='featurestore.core.CountCaseFilter.text', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='numeric', full_name='featurestore.core.CountCaseFilter.numeric', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='boolean', full_name='featurestore.core.CountCaseFilter.boolean', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='operator', full_name='featurestore.core.CountCaseFilter.operator', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='featurestore.core.CountCaseFilter.value', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='condition', full_name='featurestore.core.CountCaseFilter.condition',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=280,
  serialized_end=501,
)


_FILTER = _descriptor.Descriptor(
  name='Filter',
  full_name='featurestore.core.Filter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='text', full_name='featurestore.core.Filter.text', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='numeric', full_name='featurestore.core.Filter.numeric', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='boolean', full_name='featurestore.core.Filter.boolean', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='count_case', full_name='featurestore.core.Filter.count_case', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='condition', full_name='featurestore.core.Filter.condition',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=504,
  serialized_end=741,
)


_QUERY = _descriptor.Descriptor(
  name='Query',
  full_name='featurestore.core.Query',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='filters', full_name='featurestore.core.Query.filters', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=743,
  serialized_end=794,
)

_COUNTCASEFILTER.fields_by_name['text'].message_type = _TEXTUALFILTER
_COUNTCASEFILTER.fields_by_name['numeric'].message_type = _NUMERICALFILTER
_COUNTCASEFILTER.fields_by_name['boolean'].message_type = _BOOLEANFILTER
_COUNTCASEFILTER.oneofs_by_name['condition'].fields.append(
  _COUNTCASEFILTER.fields_by_name['text'])
_COUNTCASEFILTER.fields_by_name['text'].containing_oneof = _COUNTCASEFILTER.oneofs_by_name['condition']
_COUNTCASEFILTER.oneofs_by_name['condition'].fields.append(
  _COUNTCASEFILTER.fields_by_name['numeric'])
_COUNTCASEFILTER.fields_by_name['numeric'].containing_oneof = _COUNTCASEFILTER.oneofs_by_name['condition']
_COUNTCASEFILTER.oneofs_by_name['condition'].fields.append(
  _COUNTCASEFILTER.fields_by_name['boolean'])
_COUNTCASEFILTER.fields_by_name['boolean'].containing_oneof = _COUNTCASEFILTER.oneofs_by_name['condition']
_FILTER.fields_by_name['text'].message_type = _TEXTUALFILTER
_FILTER.fields_by_name['numeric'].message_type = _NUMERICALFILTER
_FILTER.fields_by_name['boolean'].message_type = _BOOLEANFILTER
_FILTER.fields_by_name['count_case'].message_type = _COUNTCASEFILTER
_FILTER.oneofs_by_name['condition'].fields.append(
  _FILTER.fields_by_name['text'])
_FILTER.fields_by_name['text'].containing_oneof = _FILTER.oneofs_by_name['condition']
_FILTER.oneofs_by_name['condition'].fields.append(
  _FILTER.fields_by_name['numeric'])
_FILTER.fields_by_name['numeric'].containing_oneof = _FILTER.oneofs_by_name['condition']
_FILTER.oneofs_by_name['condition'].fields.append(
  _FILTER.fields_by_name['boolean'])
_FILTER.fields_by_name['boolean'].containing_oneof = _FILTER.oneofs_by_name['condition']
_FILTER.oneofs_by_name['condition'].fields.append(
  _FILTER.fields_by_name['count_case'])
_FILTER.fields_by_name['count_case'].containing_oneof = _FILTER.oneofs_by_name['condition']
_QUERY.fields_by_name['filters'].message_type = _FILTER
DESCRIPTOR.message_types_by_name['TextualFilter'] = _TEXTUALFILTER
DESCRIPTOR.message_types_by_name['NumericalFilter'] = _NUMERICALFILTER
DESCRIPTOR.message_types_by_name['BooleanFilter'] = _BOOLEANFILTER
DESCRIPTOR.message_types_by_name['CountCaseFilter'] = _COUNTCASEFILTER
DESCRIPTOR.message_types_by_name['Filter'] = _FILTER
DESCRIPTOR.message_types_by_name['Query'] = _QUERY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TextualFilter = _reflection.GeneratedProtocolMessageType('TextualFilter', (_message.Message,), {
  'DESCRIPTOR' : _TEXTUALFILTER,
  '__module__' : 'featurestore.core.FeatureSetSearch_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.core.TextualFilter)
  })
_sym_db.RegisterMessage(TextualFilter)

NumericalFilter = _reflection.GeneratedProtocolMessageType('NumericalFilter', (_message.Message,), {
  'DESCRIPTOR' : _NUMERICALFILTER,
  '__module__' : 'featurestore.core.FeatureSetSearch_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.core.NumericalFilter)
  })
_sym_db.RegisterMessage(NumericalFilter)

BooleanFilter = _reflection.GeneratedProtocolMessageType('BooleanFilter', (_message.Message,), {
  'DESCRIPTOR' : _BOOLEANFILTER,
  '__module__' : 'featurestore.core.FeatureSetSearch_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.core.BooleanFilter)
  })
_sym_db.RegisterMessage(BooleanFilter)

CountCaseFilter = _reflection.GeneratedProtocolMessageType('CountCaseFilter', (_message.Message,), {
  'DESCRIPTOR' : _COUNTCASEFILTER,
  '__module__' : 'featurestore.core.FeatureSetSearch_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.core.CountCaseFilter)
  })
_sym_db.RegisterMessage(CountCaseFilter)

Filter = _reflection.GeneratedProtocolMessageType('Filter', (_message.Message,), {
  'DESCRIPTOR' : _FILTER,
  '__module__' : 'featurestore.core.FeatureSetSearch_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.core.Filter)
  })
_sym_db.RegisterMessage(Filter)

Query = _reflection.GeneratedProtocolMessageType('Query', (_message.Message,), {
  'DESCRIPTOR' : _QUERY,
  '__module__' : 'featurestore.core.FeatureSetSearch_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.core.Query)
  })
_sym_db.RegisterMessage(Query)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



import os

import google.protobuf.text_format
from google.protobuf.timestamp_pb2 import Timestamp


class Utils:
    @staticmethod
    def pretty_print_proto(m):
        def printer(msg, indent, as_one_line):
            if msg.DESCRIPTOR == Timestamp.DESCRIPTOR:
                return str(msg.ToDatetime().isoformat())
            return None

        return google.protobuf.text_format.MessageToString(m, message_formatter=printer)

    @staticmethod
    def timestamp_to_string(timestamp):
        if timestamp.ByteSize() != 0:
            return timestamp.ToDatetime().isoformat()
        else:
            return None

    @staticmethod
    def read_env(variable_name, source):
        value = os.environ.get(variable_name)
        if value is None:
            raise Exception(
                "Environment variable "
                + variable_name
                + " is missing, it is required to read from "
                + source
                + " data source."
            )
        else:
            return value


def get_index_of_matching_ending_angular_bracket(string):
    stack = []
    for idx, ch in enumerate(string):
        if ch == "<":
            stack.append(idx)
        elif ch == ">":
            if len(stack) == 1:
                return idx
            else:
                stack.pop()


def extract_pair(schema):
    from .schema import Column

    splits = schema.split()
    col_name = splits[0]
    remaining_schema = " ".join(splits[1:])
    if remaining_schema.upper().startswith(
        "STRUCT"
    ) or remaining_schema.upper().startswith("ARRAY"):
        idx = get_index_of_matching_ending_angular_bracket(remaining_schema)
        data_type = remaining_schema[: idx + 1].strip()
        remaining_schema = remaining_schema[idx + 1 :].strip()
        remaining_schema = remaining_schema[1:].strip()
    else:
        data_type = remaining_schema.lstrip().split(",")[0].strip()
        remaining_schema = ",".join(remaining_schema.split(",")[1:]).strip()
    return Column(col_name, data_type), remaining_schema


def extract_columns(schema):
    if schema:
        col, remaining = extract_pair(schema)
        return [col] + extract_columns(remaining)
    else:
        return []
